@@ -29,43 +29,57 @@
 # pooledSD
 
 
+
 wideToLong <- function( data, within="within", sep="_", split=TRUE) {
-	
-	ind <- grep(sep,names(data),fixed=TRUE) # indices of variables that are repeated
-	idvar <- names(data)[-ind] # names of id variables
-	tmp <- t(as.data.frame(strsplit( names(data[ind]), sep, fixed=TRUE ))) # matrix with split var names
-	v.names <- unique(tmp[,1]) # grab the measure var names
-	times <- unique(apply( tmp[,-1,drop=FALSE],1,paste,collapse=sep)) # measure 'time' names
-	varying <- list()
-	for( i in seq_along(v.names) ) varying[[i]] <- names(data)[ind][tmp[,1]==v.names[i]] 
-	
-	tmp <-make.unique(c(names(data),"withintmp"))
-	within.tmp <- tmp[length(tmp)]
-	x<-reshape( data, idvar=idvar, varying=varying, direction="long", times=times, v.names=v.names, timevar=within.tmp )
-	
-	if( split==TRUE & length( grep(sep,times,fixed=TRUE))>0 ) { # split multiple treatments into two factors?
-		split.treatments <- t(as.data.frame(strsplit(x[,within.tmp],sep,fixed=TRUE)))
-		rownames(split.treatments)<-NULL
-		split.treatments <- as.data.frame(split.treatments)
-		if( length(within)==1) { 
-			names(split.treatments) <- paste(within,1:length(split.treatments),sep="") 
-		} else {
-			if( length(within) == length(split.treatments)) {
-				names(split.treatments) <- within 
-			} else { stop( "length of 'within' is incorrect" )}
-		}
-		x <- x[,setdiff(names(x),within.tmp)] # delete collapsed treatment
-		x <- cbind(x,split.treatments) # append split treatment
-	} else { 
-		x[,within.tmp]<- factor(x[,within.tmp])
-		names(x)[grep(within.tmp,names(x))] <- within 
-	}
-	rownames(x) <- NULL
-	names(x) <- make.unique(names(x))
-	return(x)
-	
+  
+  ind <- grep(sep,names(data),fixed=TRUE) # indices of variables that are repeated
+  idvar <- names(data)[-ind] # names of id variables
+  
+  # make sure that the id variables do uniquely specify cases
+  n.profiles <- dim( unique( as.matrix( data[,idvar,drop=FALSE] ), margin=1 ) )[1] # number of unique id-var profiles
+  if( n.profiles < dim(data)[1] ) { # if id variables don't uniquely specify cases
+    warning( "Between-subject variables must uniquely specify cases: a case 'id' variable has been added")
+    id <- 1:dim(data)[1]
+    data <- cbind(data,id)
+    names(data) <- make.unique(names(data))
+    ind <- grep(sep,names(data),fixed=TRUE) # indices of variables that are repeated
+    idvar <- names(data)[-ind] # names of id variables
+  }
+  
+  tmp <- t(as.data.frame(strsplit( names(data[ind]), sep, fixed=TRUE ))) # matrix with split var names
+  v.names <- unique(tmp[,1]) # grab the measure var names
+  times <- unique(apply( tmp[,-1,drop=FALSE],1,paste,collapse=sep)) # measure 'time' names
+  varying <- list()
+  for( i in seq_along(v.names) ) varying[[i]] <- names(data)[ind][tmp[,1]==v.names[i]] 
+  
+  tmp <-make.unique(c(names(data),"withintmp"))
+  within.tmp <- tmp[length(tmp)]
+  x<-reshape( data, idvar=idvar, varying=varying, direction="long", times=times, v.names=v.names, timevar=within.tmp )
+  
+  if( split==TRUE & length( grep(sep,times,fixed=TRUE))>0 ) { # split multiple treatments into two factors?
+    split.treatments <- t(as.data.frame(strsplit(x[,within.tmp],sep,fixed=TRUE)))
+    rownames(split.treatments)<-NULL
+    split.treatments <- as.data.frame(split.treatments)
+    if( length(within)==1) { 
+      names(split.treatments) <- paste(within,1:length(split.treatments),sep="") 
+    } else {
+      if( length(within) == length(split.treatments)) {
+        names(split.treatments) <- within 
+      } else { stop( "length of 'within' is incorrect" )}
+    }
+    x <- x[,setdiff(names(x),within.tmp)] # delete collapsed treatment
+    x <- cbind(x,split.treatments) # append split treatment
+  } else { 
+    x[,within.tmp]<- factor(x[,within.tmp])
+    names(x)[grep(within.tmp,names(x))] <- within 
+  }
+  rownames(x) <- NULL
+  names(x) <- make.unique(names(x))
+  return(x)
+  
 }
 
+
 longToWide <- function( data, formula, sep="_") {
 	
 	within <- all.vars(formula[-2])
@@ -170,7 +184,16 @@ posthocPairwiseT <- function(x,...) {
 }
 
 
-cohensD <- function(x, y = NULL, data = NULL, method = "pooled",  mu = 0 ) {
+cohensD <- function(x = NULL, y = NULL, data = NULL, method = "pooled",  mu = 0, formula=x ) {
+  
+  # check to see if the user has specified a formula
+  if( is.null(x) ) { 
+    if( !is.null(formula) & class(formula)=='formula' ) { 
+      x <- formula
+    } else {
+      stop("input must specify either 'x' or 'formula'")
+    }
+  }
   
   # split formula if need be...
   if (is(x,"formula")) {
@@ -185,18 +208,18 @@ cohensD <- function(x, y = NULL, data = NULL, method = "pooled",  mu = 0 ) {
     y <- x[[2]]
     x <- x[[1]]
   }
-      
+  
   if (is.null(y)) { d <- (mean(x) - mu) / sd(x) }  # one sample 
   else {  # two sample... 
     mean.diff <- mean(x) - mean(y)
     sd.est <- switch( EXPR = method,             
-          "x.sd" = sd(x),
-          "y.sd" = sd(y),
-          "pooled" = pooledSD(x,y),
-          "corrected" = pooledSD(x,y),
-          "raw" = pooledSD(x,y,FALSE),
-          "paired" = sd(x-y),
-          "unequal" = sqrt( (var(x)+var(y))/2 )
+                      "x.sd" = sd(x),
+                      "y.sd" = sd(y),
+                      "pooled" = pooledSD(x,y),
+                      "corrected" = pooledSD(x,y),
+                      "raw" = pooledSD(x,y,FALSE),
+                      "paired" = sd(x-y),
+                      "unequal" = sqrt( (var(x)+var(y))/2 )
     )               
     d <- mean.diff / sd.est
     if( method == "corrected") { 
@@ -207,6 +230,7 @@ cohensD <- function(x, y = NULL, data = NULL, method = "pooled",  mu = 0 ) {
   return(abs(d))
 }
 
+
                       
 pooledSD <- function(x,y,debias = TRUE) {  
   sq.devs <- (c( x-mean(x), y-mean(y) ))^2
@@ -220,38 +244,46 @@ pooledSD <- function(x,y,debias = TRUE) {
 etaSquared<- function( x, type = 2, anova = FALSE ) {
   
   if( type == 1) {
-      
+    
     ss <- anova(x)[,"Sum Sq",drop=FALSE]  # Type 1 SS 
     ss.res <- ss[dim(ss)[1],]  # Full model RSS
     ss.tot <- sum( ss )  # Total SS 
     ss <- ss[-dim(ss)[1],,drop=FALSE]
+    ss <- as.matrix(ss) # later code assumes ss is a matrix
     
   } else { if (type == 2) {
-      
+    
+    # get the residual and total sum of squares
     ss.tot <- sum(( x$model[,1] - mean(x$model[,1]) )^2)
     ss.res <- sum(( x$residuals)^2)
-    t <- attr(x$terms,"term.labels")
-    ord <- attr(x$terms,"order")
-    max.ord <- max(ord)
-    ss <- matrix(NA,length(t),1)
-    rownames(ss) <- t
+    
+    # get information about how terms depend on variables (1st row is the DV, so drop it)
+    terms <- attr(x$terms,"factors")[-1,,drop=FALSE] 
+    
+    # initialise the ss matrix
+    l <- attr(x$terms,"term.labels")
+    ss <- matrix(NA,length(l),1)
+    rownames(ss) <- l
+    
+    # compute ss values
     for( i in seq_along(ss) ) {
       
-      # find the i-th term plus its higher order terms
-      if( length( grep(":",t[i]) )>0 ) {
-        ti <- seq_along(t) 
-        vi <- unlist(strsplit(t[i],":"))
-        for( v in vi ) ti <- intersect(ti, grep(v,t)) 
-      } else {
-        ti <- grep(t[i],t)
-      }
-    
-      # now calculate the corresponding ss value
-      m0 <- lm( x$terms[-ti], x$model )  # remove all of these
-      if( ord[i] < max.ord) {
-        m1 <- lm( x$terms[-setdiff(ti,i)], x$model ) # remove all except i-th term
+      # what variables does this term depend on?
+      vars.this.term <- which( terms[,i] != 0 ) 
+      
+      # which terms are dependent on this term?
+      dependent.terms <- which( apply( terms[ vars.this.term,,drop=FALSE], 2, prod )>0 ) 
+      
+      # null model removes all of the dependent terms
+      m0 <- lm( x$terms[-dependent.terms], x$model )  # remove all of these
+      
+      # terms with higher order terms need a separate alternative model...
+      if( length(dependent.terms)>1 ) {
+        m1 <- lm( x$terms[-setdiff(dependent.terms,i)], x$model ) # remove all except i-th term
         ss[i] <- anova(m0,m1)$`Sum of Sq`[2] # get the ss value
-      } else {
+        
+        # terms without higher order dependent terms can be directly compared to the full model...  
+      } else { 
         ss[i] <- anova(m0,x)$`Sum of Sq`[2]
       }
     }
@@ -259,22 +291,25 @@ etaSquared<- function( x, type = 2, anova = FALSE ) {
     
   } else { if (type == 3) { 
     
-    ss <- drop1(x,scope=x$terms)[-1,"Sum of Sq",drop=FALSE] # Type 3 SS
-    ss.res <- ss[dim(ss)[1],]  # Full model RSS
+    mod <- drop1(x,scope=x$terms)
+    ss <- mod[-1,"Sum of Sq",drop=FALSE] # Type 3 SS
+    ss.res <- mod[1,"RSS"] # residual SS
     ss.tot <- sum(( x$model[,1] - mean(x$model[,1]) )^2)
+    ss <- as.matrix(ss) # later code assumes ss is a matrix
     
   } else { 
     stop("type must be equal to 1,2 or 3") 
   }}} 
   
-  
+  # output matrix if anova not requested...
   if( anova == FALSE) {
     eta2 <- ss / ss.tot
     eta2p <- ss / (ss + ss.res)
     E <- cbind(eta2, eta2p)
     rownames(E) <- rownames(ss)
     colnames(E) <- c("eta.sq","eta.sq.part")
-  
+    
+    # output matrix if anova is requested...  
   } else {
     ss <- rbind( ss, ss.res )
     eta2 <- ss / ss.tot
@@ -292,7 +327,7 @@ etaSquared<- function( x, type = 2, anova = FALSE ) {
     rownames(E)[k] <- "Residuals"
   }
   return(E)
-
+  
 }
 
 
